#ifndef _rfsv_h_
#define _rfsv_h_

class ppsocket;
class bufferStore;
class bufferArray;

#define RFSV_SENDLEN 230

// Abstract base class of RFSV ; 16-bit and 32-bit variants implement this
// interface
class rfsv {
public:
  virtual ~rfsv() {}
  virtual void reset() = 0;
  virtual void reconnect() = 0;
  virtual long getStatus() = 0;
  virtual const char *getConnectName() = 0;
  virtual long fopen(long attr, const char *name, long &handle) = 0;
  virtual long mktemp(long *handle, char *tmpname) = 0;
  virtual long fcreatefile(long attr, const char *name, long &handle) = 0;
  virtual long freplacefile(long attr, const char *name, long &handle) = 0;
  virtual long fopendir(long attr, const char *name, long &handle) = 0;
  virtual long fclose(long handle) = 0;
  virtual long dir(const char* psionName, bufferArray *files) = 0;
  virtual long fgetmtime(const char *name, long *mtime) = 0;
  virtual long fsetmtime(const char *name, long mtime) = 0;
  virtual long fgetattr(const char *name, long *attr) = 0;
  virtual long fgeteattr(const char *name, long *attr, long *size, long *time) =0; 
  virtual long fsetattr(const char *name, long seta, long unseta) = 0;
  virtual long dircount(const char *name, long *count) = 0;
  virtual long devlist(long *devbits) = 0;
  virtual char *devinfo(int devnum, long *vfree, long *vtotal, long *vattr, long *vuiqueid) = 0;
  virtual char *opErr(long status) = 0;
  virtual long fread(long handle, char *buf, long len) = 0;
  virtual long fwrite(long handle, char *buf, long len) = 0;
  virtual long copyFromPsion(const char *from, const char *to) = 0;
  virtual long copyToPsion(const char *from, const char *to) = 0;
  virtual long fsetsize(long handle, long size) = 0;
  virtual long fseek(long handle, long pos, long mode) = 0;
  virtual long mkdir(const char* dirName) = 0;
  virtual long rmdir(const char *name) = 0;
  virtual long rename(const char *oldname, const char *newname) = 0;
  virtual long remove(const char *name) = 0;
};

#endif

